'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});

var _sourceMapSupport2 = require('source-map-support');

var _slicedToArray = function () { function sliceIterator(arr, i) { var _arr = []; var _n = true; var _d = false; var _e = undefined; try { for (var _i = arr[Symbol.iterator](), _s; !(_n = (_s = _i.next()).done); _n = true) { _arr.push(_s.value); if (i && _arr.length === i) break; } } catch (err) { _d = true; _e = err; } finally { try { if (!_n && _i["return"]) _i["return"](); } finally { if (_d) throw _e; } } return _arr; } return function (arr, i) { if (Array.isArray(arr)) { return arr; } else if (Symbol.iterator in Object(arr)) { return sliceIterator(arr, i); } else { throw new TypeError("Invalid attempt to destructure non-iterable instance"); } }; }();

exports.default = function (method, endpointUri, commandInfo) {
    const command = commandInfo.command,
          ref = commandInfo.ref,
          parameters = commandInfo.parameters;
    var _commandInfo$variable = commandInfo.variables;
    const variables = _commandInfo$variable === undefined ? [] : _commandInfo$variable;


    return function (...args) {
        let endpoint = endpointUri; // clone endpointUri in case we change it
        const commandParams = [...variables.map(v => Object.assign(v, {
            /**
             * url variables are:
             */
            required: true, // always required as they are part of the endpoint
            type: 'string' // have to be always type of string
        })), ...parameters];

        const commandUsage = `${command}(${commandParams.map(p => p.name).join(', ')})`;
        const moreInfo = `\n\nFor more info see ${ref}\n`;
        const body = {};

        /**
         * parameter check
         */
        const minAllowedParams = commandParams.filter(param => param.required).length;
        if (args.length < minAllowedParams || args.length > commandParams.length) {
            const parameterDescription = commandParams.length ? `\n\nProperty Description:\n${commandParams.map(p => `  "${p.name}" (${p.type}): ${p.description}`).join('\n')}` : '';

            throw new Error(`Wrong parameters applied for ${command}\n` + `Usage: ${commandUsage}` + parameterDescription + moreInfo);
        }

        /**
         * parameter type check
         */
        var _iteratorNormalCompletion = true;
        var _didIteratorError = false;
        var _iteratorError = undefined;

        try {
            for (var _iterator = Object.entries(args)[Symbol.iterator](), _step; !(_iteratorNormalCompletion = (_step = _iterator.next()).done); _iteratorNormalCompletion = true) {
                const _ref = _step.value;

                var _ref2 = _slicedToArray(_ref, 2);

                const i = _ref2[0];
                const arg = _ref2[1];

                const commandParam = commandParams[i];

                if (!(0, _utils.isValidParameter)(arg, commandParam.type)) {
                    /**
                     * ignore if argument is not required
                     */
                    if (typeof arg === 'undefined' && !commandParam.required) {
                        continue;
                    }

                    throw new Error(`Malformed type for "${commandParam.name}" parameter of command ${command}\n` + `Expected: ${commandParam.type}\n` + `Actual: ${typeof arg}` + moreInfo);
                }

                /**
                 * inject url variables
                 */
                if (i < variables.length) {
                    endpoint = endpoint.replace(`:${commandParams[i].name}`, arg);
                    continue;
                }

                /**
                 * rest of args are part of body payload
                 */
                body[commandParams[i].name] = arg;
            }
        } catch (err) {
            _didIteratorError = true;
            _iteratorError = err;
        } finally {
            try {
                if (!_iteratorNormalCompletion && _iterator.return) {
                    _iterator.return();
                }
            } finally {
                if (_didIteratorError) {
                    throw _iteratorError;
                }
            }
        }

        const request = new _request2.default(method, endpoint, body);
        this.emit('command', { method, endpoint, body });
        return request.makeRequest(this.options, this.sessionId).then(result => {
            if (result.value) {
                log.info('RESULT', result.value);
            }

            this.emit('result', { method, endpoint, body, result });
            return result.value;
        });
    };
};

var _wdioLogger = require('wdio-logger');

var _wdioLogger2 = _interopRequireDefault(_wdioLogger);

var _request = require('./request');

var _request2 = _interopRequireDefault(_request);

var _utils = require('./utils');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

(0, _sourceMapSupport2.install)();


const log = (0, _wdioLogger2.default)('webdriver');

module.exports = exports['default'];