'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});

var _sourceMapSupport2 = require('source-map-support');

var _slicedToArray = function () { function sliceIterator(arr, i) { var _arr = []; var _n = true; var _d = false; var _e = undefined; try { for (var _i = arr[Symbol.iterator](), _s; !(_n = (_s = _i.next()).done); _n = true) { _arr.push(_s.value); if (i && _arr.length === i) break; } } catch (err) { _d = true; _e = err; } finally { try { if (!_n && _i["return"]) _i["return"](); } finally { if (_d) throw _e; } } return _arr; } return function (arr, i) { if (Array.isArray(arr)) { return arr; } else if (Symbol.iterator in Object(arr)) { return sliceIterator(arr, i); } else { throw new TypeError("Invalid attempt to destructure non-iterable instance"); } }; }();

exports.default = WebDriver;

var _events = require('events');

var _wdioLogger = require('wdio-logger');

var _wdioLogger2 = _interopRequireDefault(_wdioLogger);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

(0, _sourceMapSupport2.install)();


const SCOPE_TYPES = {
    'browser': function Browser() {},
    'element': function Element() {}
};

function WebDriver(options, modifier, propertiesObject = {}) {
    /**
     * In order to allow named scopes for elements we have to propagate that
     * info within the `propertiesObject` object. This doesn't have any functional
     * advantages just provides better description of objects when debugging them
     */
    const scopeType = SCOPE_TYPES[propertiesObject.scope] || SCOPE_TYPES['browser'];
    delete propertiesObject.scope;

    const prototype = Object.create(scopeType.prototype, {
        isW3C: { value: options.isW3C }
    });
    const log = (0, _wdioLogger2.default)('webdriver');

    const eventHandler = new _events.EventEmitter();
    const EVENTHANDLER_FUNCTIONS = Object.getPrototypeOf(eventHandler);

    /**
     * WebDriver monad
     */
    function unit(sessionId, commandWrapper) {
        propertiesObject.commandList = { value: Object.keys(propertiesObject) };
        propertiesObject.options = { value: options

            /**
             * allow to wrap commands if necessary
             * e.g. in wdio-cli to make them synchronous
             */
        };if (typeof commandWrapper === 'function') {
            var _iteratorNormalCompletion = true;
            var _didIteratorError = false;
            var _iteratorError = undefined;

            try {
                for (var _iterator = Object.entries(propertiesObject)[Symbol.iterator](), _step; !(_iteratorNormalCompletion = (_step = _iterator.next()).done); _iteratorNormalCompletion = true) {
                    const _ref = _step.value;

                    var _ref2 = _slicedToArray(_ref, 2);

                    const commandName = _ref2[0];
                    const value = _ref2[1].value;

                    if (typeof value !== 'function') {
                        continue;
                    }

                    propertiesObject[commandName].value = commandWrapper(commandName, value);
                }
            } catch (err) {
                _didIteratorError = true;
                _iteratorError = err;
            } finally {
                try {
                    if (!_iteratorNormalCompletion && _iterator.return) {
                        _iterator.return();
                    }
                } finally {
                    if (_didIteratorError) {
                        throw _iteratorError;
                    }
                }
            }
        }

        let client = Object.create(prototype, propertiesObject);
        client.sessionId = sessionId;

        /**
         * register capabilities only to browser scope
         */
        if (scopeType.name === 'Browser') {
            client.capabilities = options.capabilities;
        }

        if (typeof modifier === 'function') {
            client = modifier(client, options);
        }

        client.addCommand = function (name, func) {
            unit.lift(name, func);
        };

        return client;
    }

    unit.lift = function (name, func) {
        prototype[name] = function next(...args) {
            const client = unit(this.sessionId);
            log.info('COMMAND', `${name}(${args.join(', ')})`);

            /**
             * set name of function for better error stack
             */
            Object.defineProperty(func, 'name', { writable: true });
            func.name = name;

            return func.apply(client, args);
        };
    };

    /**
     * register event emitter
     */
    for (let eventCommand in EVENTHANDLER_FUNCTIONS) {
        prototype[eventCommand] = function (...args) {
            eventHandler[eventCommand](...args);
            return this;
        };
    }

    return unit;
}
module.exports = exports['default'];